import 'package:flutter/foundation.dart';
import 'package:twipe_flutter_core/utils/field/field.dart';
import 'package:twipe_flutter_core/utils/field/field_validator.dart';

/// This class Represent much other classes
abstract class JSONObject {
  /// JSON Data
  @protected
  Map<String, dynamic> _data;

  /// Required Fields will be checked in validation
  @protected
  List<String> get validator;

  ///  `data` must be in JSON (`Map<String, dynamic>`)
  JSONObject(this._data);

  Map<String, dynamic> getData() {
    return _data;
  }

  /// Returns Map with specified Keys in `List<String> keys`
  Map<String, dynamic> getOnly(List<String> keys) {
    Map<String, dynamic> result = {};
    for (String key in keys) {
      result[key] = _data[key];
    }
    return result;
  }

  /// Set Value in Data
  setValue(String key, dynamic value) {
    _data[key] = value;
  }

  /// Set Data
  setData(Map<String, dynamic> data) {
    _data = data;
  }

  /// Validates Data
  @mustCallSuper
  bool validate({List<String>? optionalValidator}) {
    if (optionalValidator != null) {
      return FieldValidator.validate(optionalValidator, getData()) &&
          FieldValidator.validate(validator, getData());
    }
    return FieldValidator.validate(validator, getData());
  }

  /// Return String Value from Data
  String getStringValue(String key, {String defaultValue = ''}) {
    return Field.getString(getData()[key], defaultValue);
  }

  /// Return Int Value from Data
  int getIntValue(String key, {int defaultValue = 0}) {
    return Field.getInt(getData()[key], defaultValue);
  }

  /// Return Bool Value from Data
  bool getBoolValue(String key, {bool defaultValue = false}) {
    return Field.getBool(getData()[key], defaultValue);
  }

  /// Return Double Value from Data
  double getDoubleValue(String key, {double defaultValue = 0}) {
    return Field.getDouble(getData()[key], defaultValue);
  }
}
